//Sample using LiquidCrystal library
#include <LiquidCrystal.h>

// pins config
#define LIGHTNING_TRIGGER_ANALOG_PIN 1
#define CAM_TRIGGER_ANALOG_PIN 13
#define CAM_FOCUS_ANALOG_PIN 12
#define LED_SHUTTER_ANALOG_PIN 11

#define LCD_BUTTONS_ANALOG_PIN 0
// select the pins used on the LCD panel
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);



// this is the current second count
long currentSec = 0;
long currentPicture = 0;
// this is how long the shutter switch will be on
long shutterSec  = 100; 

int lightningVal = 0;
int lightningPreviousVal = 0;

// define some values used to store the current status
boolean running = false;
#define HOME_SCREEN 0
#define INTERVAL_SCREEN 1
#define LIGHTING_SCREEN 2
int currentScreen = HOME_SCREEN;

int currentLine = 1;
int interval = 10;
int threshhold = 50; // light sensor
#define MODE_TIMER 1
#define MODE_LIGHTING 2
int currentMode  = MODE_TIMER;


// define some values used by the panel and buttons
int adc_key_in  = 0;
#define BTN_RIGHT  0
#define BTN_UP     1
#define BTN_DOWN   2
#define BTN_LEFT   3
#define BTN_SELECT 4
#define BTN_NONE   5
int lcd_key = BTN_NONE;

// read the buttons
int read_LCD_buttons()
{
  adc_key_in = analogRead(LCD_BUTTONS_ANALOG_PIN);      // read the value from the sensor 
  // my buttons when read are centered at these valies: 0, 144, 329, 504, 741
  // we add approx 50 to those values and check to see if we are close
  if (adc_key_in > 1000) return BTN_NONE; // We make this the 1st option for speed reasons since it will be the most likely result
  // For V1.1 us this threshold
  if (adc_key_in < 50)   return BTN_RIGHT;  
  if (adc_key_in < 150)  return BTN_UP; 
  if (adc_key_in < 350)  return BTN_DOWN; 
  if (adc_key_in < 550)  return BTN_LEFT; 
  if (adc_key_in < 750)  return BTN_SELECT;  

  // For V1.0 comment the other threshold and use the one below:
  /*
 if (adc_key_in < 50)   return BTN_RIGHT;  
   if (adc_key_in < 195)  return BTN_UP; 
   if (adc_key_in < 380)  return BTN_DOWN; 
   if (adc_key_in < 555)  return BTN_LEFT; 
   if (adc_key_in < 790)  return BTN_SELECT;   
   */

  return BTN_NONE;  // when all others fail, return this...
}

// Move cursor up-down in main screen
void cursorUP_DOWM() {
  if (currentScreen == HOME_SCREEN) {
    if (currentLine == 1 && lcd_key == BTN_DOWN) { 
      currentLine = 2; 
    }
    else if (currentLine == 2 && lcd_key == BTN_UP) { 
      currentLine = 1; 
    }
  }
}

// Change current option
void cursorLEFT_RIGHT() {
  if (currentScreen == HOME_SCREEN) {
    if (currentLine == 1) {
      // Change mode
      if (currentMode == MODE_TIMER && lcd_key == BTN_RIGHT) { 
        currentMode = MODE_LIGHTING; 
      }
      else if (currentMode == MODE_LIGHTING && lcd_key == BTN_LEFT){ 
        currentMode = MODE_TIMER; 
      }
    }
    else {
      // Change parameter (interval / threshhold)
      if (currentMode == MODE_TIMER) {
        if (lcd_key == BTN_LEFT)
          interval--;
        else
          interval++;
        interval = max(interval, 1);
        interval = min(interval, 999);
      } 
      else {
        if (lcd_key == BTN_LEFT)
          threshhold--;
        else
          threshhold++;
        threshhold = max(threshhold, 1);
        threshhold = min(threshhold, 100);
      }
    }		
  }
  else {
    // Secondary screen
    running = !running;
    if (!running)
      currentPicture = 0;

    lightControl(); //init lightcontrol
  }
}

void cursorSELECT() {
  if (currentScreen == HOME_SCREEN) {
    if (currentMode == MODE_TIMER)
      currentScreen = INTERVAL_SCREEN;
    else
      currentScreen = LIGHTING_SCREEN;
    currentLine = 1;
  }
  else {
    currentScreen = HOME_SCREEN;
    running = false;
    currentPicture = 0;
    currentLine = 1;
  }
}

void refreshLCD() {
  if (currentScreen == HOME_SCREEN) {
        Serial.print("HOME_SCREEN");
    // First line
    lcd.setCursor(0, 0);
    if (currentLine == 1)
      lcd.print(">");
    else
      lcd.print(" ");
    if (currentMode == MODE_TIMER)
      lcd.print("TIMER          ");
    else
      lcd.print("LIGHTING       ");

    // Second line
    lcd.setCursor(0, 1);
    if (currentLine == 2)
      lcd.print(">");
    else
      lcd.print(" ");
    if (currentMode == MODE_TIMER) {
      lcd.print("INTERVAL:");
      lcd.print(interval);
      lcd.print("sg");
      if (interval < 10) lcd.print("    ");
      else if (interval < 100) lcd.print("   ");
      else if (interval < 1000) lcd.print("   ");
    }
    else {
      lcd.print("THRESHHOLD:");
      lcd.print(threshhold);
      lcd.print("%");
      if (threshhold < 10) lcd.print("   ");
      else if (threshhold < 100) lcd.print("  ");
      else if (threshhold < 1000) lcd.print(" ");
    }
  }
  else {
    Serial.print("INTERVAL/TIMER_SCREEN");
    lcd.setCursor(0, 0);
    lcd.print(">");
    if (running)
      lcd.print("RUNNING");
    else
      lcd.print("STOPPED");

    lcd.setCursor(0, 1);
    lcd.print("                ");
    lcd.setCursor(0, 1);
    lcd.print(" PICTURES:");
    lcd.print(currentPicture);
  }
}


void cameraTrigger() {
  if (running) {
    if (currentMode == MODE_TIMER) {
      if(interval<=currentSec){

        takePicture();

        delay(1000-shutterSec);
        currentSec = 0;        
      }

      else{
        delay(1000);
        currentSec = currentSec++;
      }
    } 
    else {
      if (lightningVal - lightningPreviousVal > threshhold)
        takePicture();
    }
  }
}

void takePicture() {
  digitalWrite(LED_SHUTTER_ANALOG_PIN, HIGH);  // turn the ledPin on
  digitalWrite(CAM_TRIGGER_ANALOG_PIN, HIGH);  // turn the CAM_TRIGGER_ANALOG_PIN on

  delay(shutterSec); // keep the led and the switch on

  digitalWrite(LED_SHUTTER_ANALOG_PIN, LOW);   // turn the ledPin off
  digitalWrite(CAM_TRIGGER_ANALOG_PIN, LOW);   // turn the CAM_TRIGGER_ANALOG_PIN off

  currentPicture++;
  
  refreshLCD();
  Serial.println("Shutter triggered");
}

void readButtons() {
  lcd_key = read_LCD_buttons();  
  if (lcd_key != BTN_NONE) {
    if (lcd_key == BTN_UP || lcd_key == BTN_DOWN)
      cursorUP_DOWM();
    else if (lcd_key == BTN_LEFT || lcd_key == BTN_RIGHT)
      cursorLEFT_RIGHT();
    else
      cursorSELECT();

    refreshLCD();
    
    delay(250);
    
    Serial.println(adc_key_in);
    Serial.println("Button detected:" + lcd_key);
  }
}

void lightControl() {
  if (currentMode == MODE_LIGHTING && running) {
    lightningPreviousVal = lightningVal;
    lightningVal = analogRead(LIGHTNING_TRIGGER_ANALOG_PIN);
  }
}

void setup()
{
  Serial.begin(9600);      // abre el puerto serie a 9600 bps:
  // declare ledPin as an OUTPUTs
  pinMode(LED_SHUTTER_ANALOG_PIN, OUTPUT);

  lcd.begin(16, 2);              // start the library
  refreshLCD();

  Serial.println("Initialized");
}

void loop()
{
  // read the buttons
  readButtons();

  // lightcontrol
  lightControl();

  // camera trigger
  cameraTrigger();
}



